"""
    test_16_asset_bundle

    As a developer working with asset bundles
    I want a way to get to sub-assets.
    
"""

import secrets
import logging
import json

from starfish.asset import MemoryAsset
from starfish.asset import BundleAsset

TEST_ASSET_COUNT = 4


def test_16_asset_bundle_register(ocean, config, surfer_agent, metadata):
    pass
        
